"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @file react组件声明地址
 */
const utils_1 = require("../utils");
const resolveDefaultConfig_1 = require("../utils/resolveDefaultConfig");
const _ = require("lodash");
const path = require("path");
function resolveSource(source) {
    if (typeof source === 'string') {
        return resolveSourceFile(source);
    }
    const defaultSource = {
        base: process.cwd(),
        fileExtension: 'js',
        components: []
    };
    source = resolveDefaultConfig_1.default(defaultSource, source);
    let components = source.components;
    components.map((comp) => {
        const compLocation = _.has(comp[1], 'location') ? comp[1].location : '';
        comp[1] = {
            location: resolveCompLocation(source.base, comp[0], compLocation, source.fileExtension)
        };
    });
    return getValidComponents(components);
}
exports.default = resolveSource;
/**
 * 解析组件绝对地址, 没有则返回null
 * @param base
 * @param compName
 * @param compLocation
 * @param fileExtension
 */
function resolveCompLocation(base, compName, compLocation, fileExtension) {
    let compPath = '';
    if (compLocation) {
        if (path.isAbsolute(compLocation)) {
            compPath = compLocation;
        }
        else {
            compPath = path.join(base, compLocation);
        }
    }
    else {
        compPath = path.join(base, compName);
    }
    if (utils_1.isDirectory(compPath)) {
        compPath = path.join(compPath, `./index.${fileExtension}`);
    }
    else {
        if (!utils_1.hasExtension(compPath)) {
            compPath = `${compPath}.${fileExtension}`;
        }
    }
    if (utils_1.isFile(compPath)) {
        return compPath;
    }
    else {
        return null;
    }
}
/**
 * 得到绝对路径
 * @param source 文件地址
 */
function resolveSourceFile(source) {
    if (!path.isAbsolute(source)) {
        source = path.join(process.cwd(), source);
    }
    if (utils_1.isFile(source)) {
        return source;
    }
    else {
        throw new Error(`${source} is not a file`);
    }
}
function getValidComponents(components) {
    const validComponents = {};
    components.forEach(comp => {
        if (comp[1] && comp[1].location) {
            comp[1].cwd = path.dirname(comp[1].location);
            validComponents[comp[0]] = comp[1];
        }
    });
    return validComponents;
}
