"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const path_1 = require("path");
const path = require("path");
const t = require("babel-types");
const ignore_1 = require("../utils/ignore");
const getJsonObjComments_1 = require("../utils/getJsonObjComments");
const comments_1 = require("../core/comments");
const easyPropTypes = path.join(__dirname, '../utils/propTypes.js');
module.exports = function () {
    let dir = '';
    let propTypesSpe = null;
    return {
        visitor: {
            Program: {
                // @ts-ignore
                enter(path, state) {
                    dir = path_1.dirname(state.file.opts.filename);
                }
            },
            ImportDeclaration(path, state) {
                const sourceName = _.get(path, 'node.source.value');
                if (/(\.less|\.css|\.sass|\.png|\.jpg|\.svg|\.gif)$/.test(sourceName)) {
                    return path.remove();
                }
                if (sourceName === 'prop-types') {
                    propTypesSpe = _.get(path, 'node.specifiers.0.local.name');
                    path.get('source').replaceWith(t.stringLiteral(easyPropTypes));
                }
                else {
                    const alias = _.get(state, 'opts.alias');
                    if (!alias) {
                        return false;
                    }
                    path.get('source').replaceWith(t.stringLiteral(resolveAlias(sourceName, alias, dir)));
                }
            },
            // @ts-ignore
            ObjectExpression(path) {
                if (propTypesSpe && isPropTypesJson(path, propTypesSpe)) {
                    comments_1.addComments(getJsonObjComments_1.default(path));
                }
                else {
                    return false;
                }
            },
            ExportDefaultDeclaration: ignore_1.default
        }
    };
};
function resolveAlias(filePath, alias, dirname) {
    if (filePath.startsWith('.')) {
        return path.join(dirname, filePath);
    }
    for (let key of Object.keys(alias)) {
        if (filePath.startsWith(`${key}${path.sep}`)) {
            return path.join(alias[key], `.${filePath.substr(key.length)}`);
        }
    }
    return filePath;
}
function isPropTypesJson(path, propTypesSpe) {
    const properties = _.get(path, 'node.properties');
    if (!(properties instanceof Array)) {
        return false;
    }
    return properties.some(p => {
        const calleeName = _.get(p, 'value.callee.object.name');
        return calleeName === propTypesSpe;
    });
}
