"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * 解析类如
 * {
 *   ...arg
 *   a: A,
 *   b: B
 * }
 * 对象的依赖存在与代码中的位置
 * @param path path对象
 */
const t = require("babel-types");
const _ = require("lodash");
const findIndentifierDeclaration_1 = require("./findIndentifierDeclaration");
const ignore_1 = require("./ignore");
function findAllDependencies(path, visitedId, dependences) {
    dependences = dependences ? dependences : [];
    visitedId = visitedId ? visitedId : {};
    if (t.isIdentifier(path)) {
        const dep = findIndentifierDeclaration_1.default(path, _.get(path, 'node.name'));
        if (dep) {
            dependences.push(dep);
        }
        return dependences;
    }
    _.set(path, 'visited', true);
    path.traverse({
        ArrowFunctionExpression: ignore_1.default,
        FunctionExpression: ignore_1.default,
        SpreadElement: handleSpreadProperty.bind(null, visitedId, dependences),
        MemberExpression: handleMemberExpression.bind(null, visitedId, dependences),
        SpreadProperty: handleSpreadProperty.bind(null, visitedId, dependences),
        Property: handleProperty.bind(null, visitedId, dependences),
        ObjectExpression: (path) => {
            findAllDependencies(path, visitedId, dependences);
        }
    });
    dependences.forEach(path => {
        if (!_.get(path, 'visited')) {
            findAllDependencies(path, visitedId, dependences);
        }
    });
    return dependences;
}
exports.default = findAllDependencies;
function handleMemberExpression(visitedId, dependences, path) {
    if (t.isIdentifier(path.get('object'))) {
        const id = _.get(path, 'node.object.name');
        saveDependecies(path, id, visitedId, dependences);
    }
}
function handleSpreadProperty(visitedId, dependences, path) {
    let idName = '';
    if (t.isIdentifier(path.get('argument'))) {
        idName = _.get(path, 'node.argument.name');
        saveDependecies(path, idName, visitedId, dependences);
    }
}
function handleProperty(visitedId, dependences, path) {
    const valuePath = path.get('value');
    const valueNode = valuePath.node;
    switch (valueNode.type) {
        case 'MemberExpression':
        case 'Identifier':
            const valuePathMap = {
                MemberExpression: 'object.name',
                Identifier: 'name'
            };
            const id = _.get(valueNode, valuePathMap[valueNode.type]);
            saveDependecies(path, id, visitedId, dependences);
            break;
        case 'CallExpression':
            handleCallExpression(visitedId, dependences, valueNode, path);
            break;
        default:
            break;
    }
}
function handleCallExpression(visitedId, dependences, valueNode, path) {
    // a(b) or a.c(b)
    const calleeId = _.get(valueNode, 'callee.name') || _.get(valueNode, 'callee.object.name');
    saveDependecies(path, calleeId, visitedId, dependences);
    const ids = _.get(valueNode, 'arguments');
    for (let i = 0, len = ids.length; i < len; i++) {
        const id = ids[i].name;
        if (t.isIdentifier(ids[i])) {
            saveDependecies(path, id, visitedId, dependences);
        }
        if (t.isMemberExpression(ids[i])) {
            const id = _.get(ids[i], 'object.name');
            saveDependecies(path, id, visitedId, dependences);
        }
    }
}
function saveDependecies(path, id, visitedId, dependences) {
    if (visitedId[id] || !id) {
        return;
    }
    const foundDependencyPath = findIndentifierDeclaration_1.default(path, id);
    if (foundDependencyPath) {
        if (t.isImportDeclaration(foundDependencyPath)) {
            const specifiers = _.get(foundDependencyPath, 'node.specifiers');
            specifiers && specifiers.forEach((sp) => {
                const id = _.get(sp, 'local.name');
                visitedId[id] = true;
            });
        }
        visitedId[id] = true;
        dependences.push(foundDependencyPath);
    }
}
